/* Revision: 2.7.3 */

/******************************************************************************
* Copyright 1998-2015 NetBurner, Inc.  ALL RIGHTS RESERVED
*
*    Permission is hereby granted to purchasers of NetBurner Hardware to use or
*    modify this computer program for any use as long as the resultant program
*    is only executed on NetBurner provided hardware.
*
*    No other rights to use this program or its derivatives in part or in
*    whole are granted.
*
*    It may be possible to license this or other NetBurner software for use on
*    non-NetBurner Hardware. Contact sales@Netburner.com for more information.
*
*    NetBurner makes no representation or warranties with respect to the
*    performance of this computer program, and specifically disclaims any
*    responsibility for any damages, special or consequential, connected with
*    the use of this program.
*
* NetBurner
* 5405 Morehouse Dr.
* San Diego, CA 92121
* www.netburner.com
******************************************************************************/

/*------------------------------------------------------------------------------
 * Example to demonstrate how to generate pulses and count pulses on the
 * MCF5441x processor.
 *----------------------------------------------------------------------------*/
#include <predef.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <networkdebug.h>
#include <taskmon.h>
#include <serial.h>
#include <sim.h>
#include <pins.h>
#include <intcdefs.h>
#include <cfinter.h>
#include "webfuncs.h"

const char *AppName = "DMA Timer Pulse Generator/Counter";

const uint32_t OutputTimer = 3;    // can be 0 through 3
const uint32_t InputTimer  = 0;    // can be 0 through 3
const uint32_t InputTimerIrqLevel = 6;

uint32_t TimerCaptureRegDelta;
uint32_t NewTimerCaptureReg;
uint32_t LastTimerCaptureReg;



/*-----------------------------------------------------------------------------
 * Configure a DMA Timer to output a square wave at a specific frequency
 *-----------------------------------------------------------------------------*/
void setTimerOutputFreq(int timer, float freq)
{
    const uint16_t  dtmrPrescale    = 0x0000;   // clock input divider
    const uint16_t  dtmrOutputMode  = 0x0020;   // toggle
    const uint16_t  dtmrFreeRun     = 0x0008;   // restart timer when ref is reached
    //const uint16_t  dtmrClockSource = 0x0004; // internal bus clk / 16
    const uint16_t  dtmrClockSource = 0x0002;   // internal bus clk / 1
    const uint16_t  dtmrEnableTimer = 0x0001;

    const float clockFreq = 125000000.0;
    //const float clockDivider = 16.0;
    const float clockDivider = 1.0;

    float fRefCount =  (clockFreq / (clockDivider * (float)((dtmrPrescale>>8) + 1) * freq)) - 1;
    printf("Freq: %0.2f, Prescale: %d, fRefCount: %0.2f\r\n", freq, dtmrPrescale>>8, fRefCount);
    fRefCount /= 2.0;   // compensate for toggle
    printf("refCount = %0.2f\r\n", fRefCount);

#if( defined MOD5441X )
    iprintf("Configuring timer %d output for MOD5441x platform\r\n", timer);
    switch (timer)
    {
        case 0:
        J2[36].function(PINJ2_36_T0OUT);
        break;
        case 1:
        J2[34].function(PINJ2_34_T1OUT);
        break;
        case 2:
        J2[33].function(PINJ2_33_T2OUT);
        break;
        case 3:
        J2[32].function(PINJ2_32_T3OUT);
        break;
        default:
            iprintf("****ERROR ***** Timer %d not supported\r\n", timer);
            return;
    }
#elif (defined NANO54415)
    iprintf("Configuring timer %d output for NANO54415 platform\r\n", timer);
    switch (timer)
    {
        case 0:
        Pins[19].function(PIN_19_T0OUT);
        break;
        case 1:
        Pins[21].function(PIN_21_T1OUT);
        break;
        case 2:
        Pins[23].function(PIN_23_T2OUT);
        break;
        case 3:
        Pins[25].function(PIN_25_T3OUT);
        break;
        default:
            iprintf("****ERROR ***** Timer %d not supported\r\n", timer);
            return;
    }
#endif

    // reset timer, transition from 1 to 0
    sim2.timer[timer].tmr |= dtmrEnableTimer;
    sim2.timer[timer].tmr &= ~dtmrEnableTimer;

    // Configure timer
    sim2.timer[timer].tmr = dtmrPrescale | dtmrOutputMode | dtmrFreeRun | dtmrClockSource;
    uint32_t uiRefCount =  (uint32_t)(fRefCount + 0.5);
    sim2.timer[timer].trr = uiRefCount;
    iprintf("Reference Count set to: 0x%08lX (%ld)\r\n", uiRefCount, uiRefCount);

    sim2.timer[timer].tmr |= dtmrEnableTimer;
}


/*-----------------------------------------------------------------------------
 * Timer capture interrupt service routine
 *-----------------------------------------------------------------------------*/
INTERRUPT(timerInputCaptureISR, 0x2700)
{
//    static uint32_t NewTimerCaptureReg;
//    static uint32_t LastTimerCaptureReg;

    sim2.timer[InputTimer].ter |= 0x01;  // clear irq

    NewTimerCaptureReg   = sim2.timer[InputTimer].tcr;
    TimerCaptureRegDelta = NewTimerCaptureReg - LastTimerCaptureReg;
    LastTimerCaptureReg = NewTimerCaptureReg;
}



/*-----------------------------------------------------------------------------
 * Configure a DMA timer for input capture
 *-----------------------------------------------------------------------------*/
void setTimerInputCapture(int timer)
{
    const uint16_t  dtmrPrescale    = 0x0000;   // clock input divider
    const uint16_t  dtmrCaptureEdge = 0x0040;   // Capture edge: rising
    const uint16_t  dtmrClockSource = 0x0002;   // DTnIn
    const uint16_t  dtmrEnableTimer = 0x0001;

#if( defined MOD5441X )
    iprintf("Configuring timer %d input for MOD5441x platform\r\n", timer);
    switch (timer)
    {
        case 0:
        J2[36].function(PINJ2_36_T0IN);
        break;
        case 1:
        J2[34].function(PINJ2_34_T1IN);
        break;
        case 2:
        J2[33].function(PINJ2_33_T2IN);
        break;
        case 3:
        J2[32].function(PINJ2_32_T3IN);
        break;
        default:
            iprintf("****ERROR ***** Timer %d not supported\r\n", timer);
            return;
    }
#elif (defined NANO54415)
    iprintf("Configuring timer %d input for NANO54415 platform\r\n", timer);
    switch (timer)
    {
        case 0:
        Pins[19].function(PIN_19_T0IN);
        break;
        case 1:
        Pins[21].function(PIN_21_T1IN);
        break;
        case 2:
        Pins[23].function(PIN_23_T2IN);
        break;
        case 3:
        Pins[25].function(PIN_25_T3IN);
        break;
        default:
            iprintf("****ERROR ***** Timer %d not supported\r\n", timer);
            return;
    }
#endif

    // reset timer, transition from 1 to 0
    sim2.timer[timer].tmr |= dtmrEnableTimer;
    sim2.timer[timer].tmr &= ~dtmrEnableTimer;

    // Configure timer
    sim2.timer[timer].tmr = dtmrPrescale | dtmrCaptureEdge | dtmrClockSource;
    iprintf("Timer %d configured for input capture\r\n", timer);

    sim2.timer[timer].ter = 0x03;   // clear status

    switch (InputTimer)
    {
    case 0:
        SETUP_DMATIMER0_ISR(timerInputCaptureISR, InputTimerIrqLevel);
        break;

    case 1:
        SETUP_DMATIMER1_ISR(timerInputCaptureISR, InputTimerIrqLevel);
        break;

    case 2:
        SETUP_DMATIMER2_ISR(timerInputCaptureISR, InputTimerIrqLevel);
        break;

    case 3:
        SETUP_DMATIMER3_ISR(timerInputCaptureISR, InputTimerIrqLevel);
        break;
    }

    sim2.timer[timer].tmr |= dtmrEnableTimer;
}



/*-----------------------------------------------------------------------------
 * Display debug menu
 -----------------------------------------------------------------------------*/
void displayMenu(void)
{
    iprintf("\r\n----- Main Menu -----\r\n");
    iprintf("C - Enter 32-bit timer reference count value\r\n");
    iprintf("F - Enter output frequency (calculates timer count)\r\n");
    iprintf("M - Measure frequency\r\n");
    iprintf("T - Display current timer count\r\n");
    iprintf("? - Display Menu\r\n");
}


/*-----------------------------------------------------------------------------
 * Handle commands with a simple command dispatcher
 -----------------------------------------------------------------------------*/
void processCommand(char c)
{
	char buf[80];

    switch (toupper(c))
    {
		case 'C':
		  {
		  iprintf("Enter 32-bit count: ");
		  fgets(buf, 80, stdin);
		  uint32_t count = atol(buf);
          sim2.timer[OutputTimer].trr = count;
		  iprintf("\r\nSet reference counter to: 0x%8lX (%ld)\r\n", count, count);
		  break;
		  }

        case 'F':
          {
          iprintf("Enter Frequency: ");
          fgets(buf, 80, stdin);
          float freq = atof(buf);
          setTimerOutputFreq(OutputTimer, freq);
          printf("\r\nSet frequency %0.2f\r\n", freq);
          break;
          }

        case 'M':
          {
          printf("Frequency: %0.2f (0x%08lX)\r\n", (float)(125000000.0/(float)TimerCaptureRegDelta), TimerCaptureRegDelta);
          break;
          }

      case 'T':
    	  iprintf("Timer count: 0x%08lX, Last: 0x%08lX, New: 0x%08lX\r\n",
    	          sim2.timer[OutputTimer].tcn, LastTimerCaptureReg, NewTimerCaptureReg);
    	  break;

      default:
        displayMenu();
    }
}



/*-----------------------------------------------------------------------------
 * The main task
 -----------------------------------------------------------------------------*/
extern "C" void UserMain(void *pd)
{
    InitializeStack();

    if (EthernetIP == 0)
    {
        iprintf("Trying DHCP...\r\n");
        GetDHCPAddress();
        iprintf("DHCP assigned the IP address of:  ");
        ShowIP(EthernetIP);
        iprintf("\r\n");
    }
    else
    {
        iprintf("Static IP address:  ");
        ShowIP(EthernetIP);
        iprintf("\r\n");
    }

    OSChangePrio(MAIN_PRIO);   // Set main task priority to the default (50)
    EnableAutoUpdate();        // Enable NB application AutoUpdate support
    StartHTTP();               // Start web server
    EnableTaskMonitor();

#ifdef _DEBUG
    InitializeNetworkGDB();
    //InitializeNetworkGDB_and_Wait();
#endif

    displayMenu();

    iprintf("Initializing output timer: %d\r\n", OutputTimer);
    setTimerOutputFreq(OutputTimer, 125);

    iprintf("Initializing input capture timer: %d\r\n", InputTimer);
    setTimerInputCapture(InputTimer);

    while (1)
    {
        char c = getchar();
        processCommand(c);
    }
}
